CATEGORY_MAPPING = {
    "Data": [
        "Data Science",
        "Data Scientist",
        "Machine Learning Engineer",
        "AI",
        "Hadoop",
    ],
    "Development - Backend": [
        "Backend Developer",
        "Python Developer",
        "Java Developer",
        "DotNet Developer",
        "Full Stack Developer",
    ],
    "Development - Frontend": ["Frontend Developer", "Web Designing"],
    "Mobile Development": ["Mobile App Developer (iOS/Android)"],
    "DevOps & Cloud": ["DevOps Engineer", "Cloud Engineer"],
    "Testing": ["Testing", "Automation Testing", "ETL Developer"],
    "Engineering": ["Mechanical Engineer", "Civil Engineer", "Electrical Engineering"],
    "Security": ["Network Security Engineer"],
    "Business": ["Business Analyst", "PMO", "Operations Manager", "Sales"],
    "HR": ["HR"],
    "Design": ["Arts", "Health and fitness"],
    "Blockchain": ["Blockchain"],
    "SAP": ["SAP Developer"],
    "Database": ["Database"],
    "Legal": ["Advocate"],
}



import re
from datasets import Dataset
from transformers import AutoTokenizer
from sklearn.preprocessing import LabelEncoder

from nltk.tokenize import word_tokenize
from nltk.corpus import stopwords
import nltk

from foai_model.config import MODEL_NAME, MAX_TOKEN_LENGTH, TEST_SPLIT_SIZE
from foai_model.logger import logger
from .categories import CATEGORY_MAPPING

nltk.download("punkt", quiet=True)
nltk.download("stopwords", quiet=True)

stop_words = set(stopwords.words("english"))


def clean_resume(text):
    logger.debug("Cleaning resume text...")
    text = text.lower()
    text = re.sub(r"http\S+|www\S+|https\S+", "", text)
    text = re.sub(r"\S+@\S+", "", text)
    text = re.sub(r"\+?\d[\d -]{8,}\d", "", text)
    text = re.sub(r"\n", " ", text)
    text = re.sub(r"[^a-zA-Z\s]", "", text)
    text = re.sub(r"\s+", " ", text).strip()

    tokens = word_tokenize(text)
    tokens = [word for word in tokens if word not in stop_words]

    return " ".join(tokens)


def simplify_category(category):
    for group, values in CATEGORY_MAPPING.items():
        if category in values:
            logger.debug("Mapping category '%s' to group '%s'", category, group)
            return group
    logger.debug("No mapping found for category '%s', keeping as is.", category)
    return category


def tokenize_categories(df):
    label_encoder = LabelEncoder()
    df["labels"] = label_encoder.fit_transform(df["Category"])
    logger.info("Encoded categories into %d label(s)", df["labels"].nunique())
    return [df, label_encoder]


def clean_dataset(df):
    logger.info("Starting dataset cleaning and tokenization pipeline...")

    original_shape = df.shape
    df = df.dropna()
    logger.info(
        "Dropped %d rows with NaN values (from %s to %s)",
        original_shape[0] - df.shape[0],
        original_shape,
        df.shape,
    )

    logger.info("Simplifying category labels...")
    df["Category"] = df["Category"].apply(simplify_category)

    logger.info("Encoding category labels...")
    df, label_encoder = tokenize_categories(df)

    logger.info("Cleaning resume texts...")
    df["Resume"] = df["Resume"].apply(clean_resume)

    logger.info("Converting DataFrame to Hugging Face Dataset...")
    dataset = Dataset.from_pandas(df[["Resume", "labels"]])
    logger.info("Converted to Dataset with %d total records.", len(dataset))

    logger.info("Splitting dataset into train/test sets...")
    dataset = dataset.train_test_split(test_size=TEST_SPLIT_SIZE)
    logger.info(
        "Train size: %d, Test size: %d",
        len(dataset["train"]),
        len(dataset["test"]),
    )

    logger.info("Tokenizing resume texts...")
    tokenizer = AutoTokenizer.from_pretrained(MODEL_NAME)

    tokenized_dataset = dataset.map(
        lambda e: tokenizer(
            e["Resume"],
            truncation=True,
            padding="max_length",
            max_length=MAX_TOKEN_LENGTH,
        )
    )
    logger.info("Finished tokenization.")

    logger.info("Dataset cleaning and tokenization pipeline completed.")
    return [df, tokenized_dataset, label_encoder]



from pathlib import Path
import os
import pandas as pd
from foai_model.logger import logger

DATASETS_PATH = Path("data/datasets")


def read_datasets():
    all_dataframes = []
    expected_columns = ["Category", "Resume"]

    for filename in os.listdir(DATASETS_PATH):
        if filename.endswith(".csv"):
            path = os.path.join(DATASETS_PATH, filename)
            df = pd.read_csv(path)

            logger.info("Loaded file: %s", filename)
            logger.info("Dataset shape: %s", df.shape)

            if list(df.columns) != expected_columns:
                logger.warning(
                    "File '%s' has different columns and will be skipped.", filename
                )
                logger.warning("Expected columns: %d", expected_columns)
                logger.warning("Found columns: %s", list(df.columns))
                continue

            all_dataframes.append(df)

    if all_dataframes:
        final_dataset = pd.concat(all_dataframes, ignore_index=True)
        logger.info("Combined dataset shape: %s", final_dataset.shape)
    else:
        logger.warning("No CSV files with matching columns were loaded.")

    return final_dataset



import argparse
import pickle
import torch
from transformers import AutoTokenizer, AutoModelForSequenceClassification
from huggingface_hub import hf_hub_download
import nltk

from foai_model.config import MAX_TOKEN_LENGTH
from foai_model.preprocessing import clean_resume

nltk.download('punkt_tab')

REPO_ID = "Dar3cz3Q/foai_model"
SUBFOLDER = "checkpoint"

tokenizer = AutoTokenizer.from_pretrained(REPO_ID, subfolder=SUBFOLDER)
model = AutoModelForSequenceClassification.from_pretrained(REPO_ID, subfolder=SUBFOLDER)
model.eval()
device = torch.device("cpu")
model.to(device)

encoder_path = hf_hub_download(
    repo_id=REPO_ID, filename="label_encoder.pkl", subfolder=SUBFOLDER
)

with open(encoder_path, "rb") as f:
    label_encoder = pickle.load(f)

index_to_label = dict(enumerate(label_encoder.classes_))


def predict(text):
    inputs = tokenizer(
        text,
        return_tensors="pt",
        padding=True,
        truncation=True,
        max_length=MAX_TOKEN_LENGTH,
    )

    inputs = {k: v.to(device) for k, v in inputs.items()}

    with torch.no_grad():
        logits = model(**inputs).logits
        predicted_class_id = torch.argmax(logits, dim=1).item()

    category = index_to_label[predicted_class_id]

    return {
        "id": predicted_class_id,
        "category": category,
    }


def main():
    parser = argparse.ArgumentParser(
        description="Predict category distribution from text."
    )
    parser.add_argument("--text", type=str, required=True, help="Text to classify")

    args = parser.parse_args()
    resume = clean_resume(args.text)
    result = predict(resume)

    print("Result:", result)


if __name__ == "__main__":
    main()



import torch
from foai_model.logger import logger


def log_device_info():
    if torch.cuda.is_available():
        num_devices = torch.cuda.device_count()
        logger.info("CUDA is available. Found %d GPU(s).", num_devices)
        for i in range(num_devices):
            logger.info("GPU %d: %s", i, torch.cuda.get_device_name(i))
    else:
        logger.warning("CUDA not available. Using CPU.")
